#!/usr/bin/perl
# dnsdhcp-search-proxy-user.pl
#Copyright © [2008(S)] Novell, Inc.  All Rights Reserved.
#THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.  
#IT MAY NOT BE USED, COPIED, DISTRIBUTED, DISCLOSED, ADAPTED, PERFORMED,
#DISPLAYED, COLLECTED, COMPILED, OR LINKED WITHOUT NOVELL'S PRIOR WRITTEN CONSENT.
#USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT
#THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#NOVELL PROVIDES THE WORK "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, 
#INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#PARTICULAR PURPOSE, AND NON-INFRINGEMENT. NOVELL, THE AUTHORS OF THE WORK,
#AND THE OWNERS OF COPYRIGHT IN THE WORK ARE NOT LIABLE FOR ANY CLAIM,
#DAMAGES, OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT,
#OR OTHERWISE, ARISING FROM, OUT OF, OR IN CONNECTION WITH THE WORK OR THE 
#USE OR OTHER DEALINGS IN THE WORK.
#
# Author: Kalyan (skalyanasundaram@novell.com)
# 
#
 
# This perl script uses the NET::LDAP APIs to search for the 
# existance of a dns/dhcp proxy user and used by YaST2 installation.

use Getopt::Std;
%args=();
getopt("s:p:l:D:u:t:",\%args);

$source = $args{s}; #IP ADDRESS
$port   = $args{p};
$ssl = $args{l};   #0-nonssl, 1-ssl
$binddn = $args{D}; #admin who has rights to see the proxy user
$userfdn = $args{u}; #proxy user to be searched
$timeout = $args{t}; 
$bindpasswd=$ENV{'OES_DATA'};

if ($source eq "" || $port eq "" || $ssl eq "" || $binddn eq "" || $userfdn eq "" || $timeout eq "" )
{
    &usage();
}

if($ssl)
{
    use Net::LDAPS;
    use Net::LDAP::Util qw(ldap_error_text
			   ldap_error_name
			   ldap_error_desc
			   );                                              #SSL bind 
    use Net::LDAP::Control;
    use Net::LDAP::Constant qw( LDAP_CONTROL_MATCHEDVALS );
    
    $ctrl = Net::LDAP::Control->new(
				    type     => "2.16.840.1.113719.1.513.4.5",
				    value    => "",
				    critical => 0
				    );
    
    $ldap = Net::LDAPS->new($source,
			    port => $port,
			    control => [ $ctrl ], 
			    timeout => $timeout );
    if ($@ ne  "")
    {
	print stderr "Fatal:$@. An ldap error occurred.";
	exit 1;
    }
    
}
else
{
    use Net::LDAP;
    use Net::LDAP::Util qw(ldap_error_text
			   ldap_error_name
			   ldap_error_desc
			   );                                             # Non-SSL bind
    use Net::LDAP::Control;
    use Net::LDAP::Constant qw( LDAP_CONTROL_MATCHEDVALS );
    
    $ctrl = Net::LDAP::Control->new(
				    type     => "2.16.840.1.113719.1.513.4.5",
				    value    => "",
				    critical => 0
				    );
    
    $ldap = Net::LDAP->new($source,
			   port => $port,
			   control => [ $ctrl ],
			   timeout => $timeout );
    if ($@ ne  "")
    {
	print stderr "Fatal:$@. An ldap error occurred.";
	exit 1;
    }
}


$mesg = $ldap->bind ($binddn, password => $bindpasswd);

#------------------------------------------------------ERROR --- 
if (ldap_error_name($mesg) ne "LDAP_SUCCESS")
{
    print stderr "Fatal:";
    print stderr ldap_error_name($mesg);
    print stderr ": ";
    print stderr ldap_error_text($mesg);
    exit 1;
    
}

my $cn = &basecn( $userfdn, 1);
my $base = &basecn( $userfdn, 0);

my $mesg = $ldap->search ( base => $base,
			   filter => "(&(commonname=$cn)(objectclass=User))" );

#------------------------------------------------------ERROR ---
if (ldap_error_name($mesg) ne "LDAP_SUCCESS")
{
    print stderr $mesg;
    exit 1;
    
}

#------------------------------------------------------ERROR ---

#print stderr $mesg->count;
if ($mesg->count > 0 ) {
    exit 0;
}
else {
    exit 1;
}

sub basecn ()                                           #BREAKS THE DN INTO CN AND BASE AND RETURNS WHAT IS ASKED FOR
{
    local($zone, $b);
    ($zone, $b) = ($_[0], $_[1]);
    my @zone = split(',',$zone);
    my $cn =  @zone[0];
    my @arrcn = split('=',$cn);
    $cn =@arrcn[1];
    
    shift (@zone);
    $base = join(",",@zone);
    if ($b eq 0)
    {
	return ($base);
    }
    else
    {
	return($cn);
    }
}

sub
    usage()
{
    print "\nUsage: perl  dnsdhcp-search-proxy-user.pl [options]\n";
    print "-s                Ldap Address\n";
    print "-p                Ldap Port\n";
    print "-l                SSL option (1 for ssl port and 0 for non ssl port)\n";
    print "-D                Tree Admin DN\n";
    print "-u                Proxy User FDN to search\n";
    print "-t                Timeout value in seconds\n";
    print "WARNING!!! This script is intented to be used by the YaST2 installation.\n\n";
    exit 1;
}


